#!/usr/bin/env python
# -*- coding: utf-8 -*-

__author__ = "Dr Mohamed Salaheddine Talamali - University College London (UCL)"

import numpy as np
import math
import matplotlib.pyplot as plt
import csv
from matplotlib.backends.backend_pdf import PdfPages
import zipfile
import os
from scipy import stats
import scipy as sp
from matplotlib import rcParams

# Legend personalization
import matplotlib.patches as mpatches
import matplotlib.lines as mlines
import warnings

def binomial_confidence_interval(data,number,confidence=0.95):
	p=data
	z=1-confidence/2.0
	conf=z*math.sqrt(p*(1-p)/float(number))
	min=p-conf
	max=p+conf
	return max,min 

def mean_confidence_interval(data, confidence=0.95):
    a = 1.0*np.array(data)
    n = len(a)
    m, se = np.mean(a), sp.stats.sem(a)
    conf = se * sp.stats.t._ppf((1+confidence)/2., n-1)
    return m, m+conf, m-conf

def movingaverage(values,window):
	weigths = np.repeat(1.0, window)/window
	smas = np.convolve(values, weigths,'valid')
	extended_smas = [0] * (window-1)
	extended_smas.extend(smas)
	return extended_smas # as a numpy array


# Number of runs
numberofruns=6

# Experiment length (in seconds)
numberoftimesteps=4800

# Experiment timestep (in seconds) 
sampling_time = 0.5

# Data plotting steps (in experiment timestep)
step=10

# Fontsize
fontsize=48

# Quorum threshold
Q = 0.8

# raw data file
data_archive = zipfile.ZipFile('raw_data.zip', 'r')

# total number of options
n=3

# number of robots
N = 50 

# communication range
commrng=0.1

# robot's field of view 
opsize=0.2

datafolder = "local_communication"

filename_prefix="timePop_"+datafolder

dataarray=np.zeros((numberofruns,int(numberoftimesteps/sampling_time)+1,n+1))

file_to_save="post_processed_data/"+datafolder+"_comm.npy"

if not os.path.exists(file_to_save):
	print("Processing row date...")
	for i in np.arange(1,numberofruns+1):
		filename=datafolder+"/"+filename_prefix+"_"+str(i)+".txt"
		print("run=",i)
		csvfile=data_archive.open(filename)
		data=np.loadtxt(csvfile, delimiter='\t', usecols = (0,1,2,3,4,5,6,7,8))
		data=np.array(data)

		timestep=0
		
		dataarray[i-1,timestep,0] = timestep*sampling_time
		dataarray[i-1,timestep,1] = 0
		dataarray[i-1,timestep,2] = 0
		dataarray[i-1,timestep,3] = 50.0

		timestep=timestep+1

		for j in np.arange(2,data.shape[0]+1):

			while(data[j,0]>timestep*sampling_time):
				if(timestep>int(numberoftimesteps/sampling_time)):
					break
				dataarray[i-1,timestep,0] = timestep*sampling_time
				dataarray[i-1,timestep,1] = dataarray[i-1,timestep-1,1]
				dataarray[i-1,timestep,2] = dataarray[i-1,timestep-1,2]
				dataarray[i-1,timestep,3] = dataarray[i-1,timestep-1,3]
				timestep=timestep+1
				if(timestep>int(numberoftimesteps/sampling_time)):
					break
			if(timestep>int(numberoftimesteps/sampling_time)):
					break

			dataarray[i-1,timestep,0] = timestep*sampling_time
			dataarray[i-1,timestep,1] = data[j,2]
			dataarray[i-1,timestep,2] = data[j,3]
			dataarray[i-1,timestep,3] = data[j,4]
			timestep=timestep+1

	np.save(file_to_save,dataarray)

else:
	print("Opening data file ...")
	dataarray=np.load(file_to_save)

OP1_AVG=[]
OP1_MAX=[]
OP1_MIN=[]

OP2_AVG=[]
OP2_MAX=[]
OP2_MIN=[]

OP3_AVG=[]
OP3_MAX=[]
OP3_MIN=[]

TIME=[]

fig=plt.figure(figsize=(20,5))
plt.rc('text', usetex=True)
rcParams['font.family'] = 'sans-serif'
rcParams['font.sans-serif'] = 'Myriad Pro'
rcParams['mathtext.fontset']= "cm"

ax1 = plt.subplot(111)

for timestep in np.arange(0,dataarray.shape[1],4):

	TIME.append(timestep*sampling_time/60.0)

	OP1=dataarray[:,timestep,1]/N
	OP1_mean, OP1_max, OP1_min = mean_confidence_interval(OP1)
	OP1_AVG.append(OP1_mean)
	OP1_MAX.append(OP1_max)
	OP1_MIN.append(OP1_min)

	OP2=dataarray[:,timestep,2]/N
	OP2_mean, OP2_max, OP2_min = mean_confidence_interval(OP2)
	OP2_AVG.append(OP2_mean)
	OP2_MAX.append(OP2_max)
	OP2_MIN.append(OP2_min)

	OP3=dataarray[:,timestep,3]/N
	OP3_mean, OP3_max, OP3_min = mean_confidence_interval(OP3)
	OP3_AVG.append(OP3_mean)
	OP3_MAX.append(OP3_max)
	OP3_MIN.append(OP3_min)



plot_linewidth=5
lines_linewidth=3

plt.plot(TIME,OP3_AVG,c='b',linestyle='-',linewidth=plot_linewidth)
plt.plot(TIME,OP2_AVG,c='g',linestyle='-',linewidth=plot_linewidth)
plt.plot(TIME,OP1_AVG,c='r',linestyle='-',linewidth=plot_linewidth)

for i in np.arange(0,numberofruns):
	TTIME=dataarray[i,:,0]/60.0
	OP1=dataarray[i,:,1]/N
	OP2=dataarray[i,:,2]/N	
	OP3=dataarray[i,:,3]/N

	plt.plot(TTIME,OP1,color='red',alpha=0.2)
	plt.plot(TTIME,OP2,color='green',alpha=0.2)
	plt.plot(TTIME,OP3,color='blue',alpha=0.2)

ax1.axvline(x=20,linewidth=lines_linewidth, color='black',linestyle='--')
ax1.axvline(x=40,linewidth=lines_linewidth, color='black',linestyle='--')
ax1.axhline(y=0.8,linewidth=lines_linewidth, color='black',linestyle='--')

x_max=80.2
x_min=-0.2
y_max=1.01
y_min=-0.01

plt.xlim((x_min,x_max))
plt.ylim((y_min,y_max))

ax1.set_ylabel("Population $(\%)$",fontsize=fontsize)
ax1.set_yticks(np.arange(0,1.01,0.2))
ax1.set_yticklabels(("0","20","40","60","80","100"),fontsize=fontsize)

ax1.set_xlabel("Time $(min)$",fontsize=fontsize)
ax1.set_xticks(np.arange(0,81,5))
ax1.set_xticklabels(np.arange(0,int(81),5),fontsize=fontsize-6)

for axis in ['top','bottom','left','right']:
	ax1.spines[axis].set_linewidth(4)

ax1.tick_params('both', length=8, width=4, which='major')
ax1.tick_params('both', length=6, width=2, which='minor')

line = mlines.Line2D([20,20], [1.0,1.3],linewidth=lines_linewidth, color='black',linestyle='--')
line.set_clip_on(False)
plt.gca().add_line(line)

line = mlines.Line2D([40,40], [1.0,1.3],linewidth=lines_linewidth, color='black',linestyle='--')
line.set_clip_on(False)
plt.gca().add_line(line)


plt.tight_layout()
pp = PdfPages("Fig5_A_no_overlay.pdf")
pp.savefig(bbox_inches='tight')
pp.close()
plt.clf()
